@@ -48,7 +48,7 @@ except ImportError:
 import sys
 import os
 import time
-import configobj
+import collections
 import subprocess
 
 import tuned.logs
@@ -80,7 +80,7 @@ AUTHORS = [
 class Base(object):
 
 	"""
-	GUI class for program Tuned.
+	GUI class for program TuneD.
 	"""
 
 	is_admin = False
@@ -102,8 +102,8 @@ class Base(object):
 				self._gobj('tunedDaemonExceptionDialog').hide()
 				return True
 			else:
-				self.error_dialog('Tuned is shutting down.',
-								  'Reason: missing communication with Tuned daemon.'
+				self.error_dialog('TuneD is shutting down.',
+								  'Reason: missing communication with TuneD daemon.'
 								  )
 				return False
 		return True
@@ -206,7 +206,7 @@ class Base(object):
 
 	def is_tuned_connection_ok(self):
 		"""
-		Result True, False depends on if tuned daemon is running. If its not runing this method try to start tuned.
+		Result True, False depends on if tuned daemon is running. If its not running this method try to start tuned.
 		"""
 
 		try:
@@ -236,7 +236,7 @@ class Base(object):
 		This add rows to object listbox_summary_of_active_profile.
 		Row consist of grid. Inside grid on first possition is label, second possition is vertical grid.
 		label = name of plugin
-		verical grid consist of labels where are stored values for plugin option and value.
+		vertical grid consist of labels where are stored values for plugin option and value.
 
 		This method is emited after change profile and on startup of app.
 		"""
@@ -356,14 +356,14 @@ class Base(object):
 		try:
 			if self._get_active_profile_name() == profile:
 				self.error_dialog('You can not remove active profile',
-								  'Please deactivate profile by choosind another!'
+								  'Please deactivate profile by choosing another!'
 								  )
 				return
 			if profile is None:
 				self.error_dialog('No profile selected!', '')
 				return
-			if self._gobj('windowProfileEditor').is_active():
-				self.error_dialog('You are ediding '
+			if self._gobj('windowProfileEditor').is_visible():
+				self.error_dialog('You are editing '
 								  + self.editing_profile_name
 								  + ' profile.',
 								  'Please close edit window and try again.'
@@ -508,8 +508,7 @@ class Base(object):
 
 	def data_to_profile_config(self):
 		name = self._gobj('entryProfileName').get_text()
-		config = configobj.ConfigObj(list_values = False,
-				interpolation = False)
+		config = collections.OrderedDict()
 
 		activated = self._gobj('comboboxIncludeProfile').get_active()
 		model = self._gobj('comboboxIncludeProfile').get_model()
@@ -706,7 +705,7 @@ class Base(object):
 				self.is_tuned_connection_ok()
 			else:
 				self._su_execute(['service', 'tuned', 'stop'])
-				self.error_dialog('Tuned Daemon is turned off',
+				self.error_dialog('TuneD Daemon is turned off',
 								  'Support of tuned is not running.')
 		elif switch == self._gobj('switchTunedStartupStartStop'):
 
