@@ -11,7 +11,78 @@ log = tuned.logs.get()
 
 class DiskPlugin(hotplug.Plugin):
 	"""
-	Plugin for tuning options of disks.
+	`disk`::
+	
+	Plug-in for tuning various block device options. This plug-in can also
+	dynamically change the advanced power management and spindown timeout
+	setting for a drive according to the current drive utilization. The
+	dynamic tuning is controlled by the [option]`dynamic` and the global
+	[option]`dynamic_tuning` option in `tuned-main.conf`.
+	+
+	The disk plug-in operates on all supported block devices unless a
+	comma separated list of [option]`devices` is passed to it.
+	+
+	.Operate only on the sda block device
+	====
+	----
+	[disk]
+	# Comma separated list of devices, all devices if commented out.
+	devices=sda
+	----
+	====
+	+
+	The [option]`elevator` option sets the Linux I/O scheduler.
+	+
+	.Use the bfq I/O scheduler on xvda block device
+	====
+	----
+	[disk]
+	device=xvda
+	elevator=bfq
+	----
+	====
+	+
+	The [option]`scheduler_quantum` option only applies to the CFQ I/O
+	scheduler. It defines the number of I/O requests that CFQ sends to
+	one device at one time, essentially limiting queue depth. The default
+	value is 8 requests. The device being used may support greater queue
+	depth, but increasing the value of quantum will also increase latency,
+	especially for large sequential write work loads.
+	+
+	The [option]`apm` option sets the Advanced Power Management feature
+	on drives that support it. It corresponds to using the `-B` option of
+	the `hdparm` utility. The [option]`spindown` option puts the drive
+	into idle (low-power) mode, and also sets the standby (spindown)
+	timeout for the drive. It corresponds to using `-S` option of the
+	`hdparm` utility.
+	+
+	.Use a medium-agressive power management with spindown
+	====
+	----
+	[disk]
+	apm=128
+	spindown=6
+	----
+	====
+	+
+	The [option]`readahead` option controls how much extra data the
+	operating system reads from disk when performing sequential
+	I/O operations. Increasing the `readahead` value might improve
+	performance in application environments where sequential reading of
+	large files takes place. The default unit for readahead is KiB. This
+	can be adjusted to sectors by specifying the suffix 's'. If the
+	suffix is specified, there must be at least one space between the
+	number and suffix (for example, `readahead=8192 s`).
+	+
+	.Set the `readahead` to 4MB unless already set to a higher value
+	====
+	----
+	[disk]
+	readahead=>4096
+	----
+	====
+	The disk readahead value can be multiplied by the constant
+	specified by the [option]`readahead_multiply` option.
 	"""
 
 	def __init__(self, *args, **kwargs):
@@ -27,22 +98,42 @@ class DiskPlugin(hotplug.Plugin):
 	def _init_devices(self):
 		super(DiskPlugin, self)._init_devices()
 		self._devices_supported = True
+		self._use_hdparm = True
 		self._free_devices = set()
+		self._hdparm_apm_devices = set()
 		for device in self._hardware_inventory.get_devices("block"):
 			if self._device_is_supported(device):
 				self._free_devices.add(device.sys_name)
+				if self._use_hdparm and self._is_hdparm_apm_supported(device.sys_name):
+					self._hdparm_apm_devices.add(device.sys_name)
 
 		self._assigned_devices = set()
 
 	def _get_device_objects(self, devices):
 		return [self._hardware_inventory.get_device("block", x) for x in devices]
 
+	def _is_hdparm_apm_supported(self, device):
+		(rc, out, err_msg) = self._cmd.execute(["hdparm", "-C", "/dev/%s" % device], \
+				no_errors = [errno.ENOENT], return_err=True)
+		if rc == -errno.ENOENT:
+			log.warn("hdparm command not found, ignoring for other devices")
+			self._use_hdparm = False
+			return False
+		elif rc:
+			log.info("Device '%s' not supported by hdparm" % device)
+			log.debug("(rc: %s, msg: '%s')" % (rc, err_msg))
+			return False
+		elif "unknown" in out:
+			log.info("Driver for device '%s' does not support apm command" % device)
+			return False
+		return True
+
 	@classmethod
 	def _device_is_supported(cls, device):
 		return  device.device_type == "disk" and \
 			device.attributes.get("removable", None) == b"0" and \
 			(device.parent is None or \
-					device.parent.subsystem in ["scsi", "virtio", "xen"])
+					device.parent.subsystem in ["scsi", "virtio", "xen", "nvme"])
 
 	def _hardware_events_init(self):
 		self._hardware_inventory.subscribe(self, "block", self._hardware_events_callback)
@@ -142,6 +233,8 @@ class DiskPlugin(hotplug.Plugin):
 		return not "standby" in out and not "sleeping" in out
 
 	def _instance_update_dynamic(self, instance, device):
+		if not device in self._hdparm_apm_devices:
+			return
 		load = instance._load_monitor.get_device_load(device)
 		if load is None:
 			return
@@ -202,7 +295,7 @@ class DiskPlugin(hotplug.Plugin):
 		diff = [new_old[0] - new_old[1] for new_old in zip(new_load, old_load)]
 		instance._stats[device]["diff"] = diff
 
-		# adapt maximum expected load if the difference is higer
+		# adapt maximum expected load if the difference is higher
 		old_max_load = instance._stats[device]["max"]
 		max_load = [max(pair) for pair in zip(old_max_load, diff)]
 		instance._stats[device]["max"] = max_load
@@ -219,6 +312,15 @@ class DiskPlugin(hotplug.Plugin):
 			else:
 				instance._idle[device][operation] = 0
 
+	def _instance_apply_dynamic(self, instance, device):
+		# At the moment we support dynamic tuning just for devices compatible with hdparm apm commands
+		# If in future will be added new functionality not connected to this command,
+		# it is needed to change it here
+		if device not in self._hdparm_apm_devices:
+			log.info("There is no dynamic tuning available for device '%s' at time" % device)
+		else:
+			super(DiskPlugin, self)._instance_apply_dynamic(instance, device)
+
 	def _instance_unapply_dynamic(self, instance, device):
 		pass
 
@@ -248,6 +350,12 @@ class DiskPlugin(hotplug.Plugin):
 
 	@command_set("apm", per_device=True)
 	def _set_apm(self, value, device, sim):
+		if device not in self._hdparm_apm_devices:
+			if not sim:
+				log.info("apm option is not supported for device '%s'" % device)
+				return None
+			else:
+				return str(value)
 		if self._apm_errcnt < consts.ERROR_THRESHOLD:
 			if not sim:
 				(rc, out) = self._cmd.execute(["hdparm", "-B", str(value), "/dev/" + device], no_errors = [errno.ENOENT])
@@ -258,6 +366,10 @@ class DiskPlugin(hotplug.Plugin):
 
 	@command_get("apm")
 	def _get_apm(self, device, ignore_missing=False):
+		if device not in self._hdparm_apm_devices:
+			if not ignore_missing:
+				log.info("apm option is not supported for device '%s'" % device)
+			return None
 		value = None
 		err = False
 		(rc, out) = self._cmd.execute(["hdparm", "-B", "/dev/" + device], no_errors = [errno.ENOENT])
@@ -278,6 +390,12 @@ class DiskPlugin(hotplug.Plugin):
 
 	@command_set("spindown", per_device=True)
 	def _set_spindown(self, value, device, sim):
+		if device not in self._hdparm_apm_devices:
+			if not sim:
+				log.info("spindown option is not supported for device '%s'" % device)
+				return None
+			else:
+				return str(value)
 		if self._spindown_errcnt < consts.ERROR_THRESHOLD:
 			if not sim:
 				(rc, out) = self._cmd.execute(["hdparm", "-S", str(value), "/dev/" + device], no_errors = [errno.ENOENT])
@@ -288,6 +406,10 @@ class DiskPlugin(hotplug.Plugin):
 
 	@command_get("spindown")
 	def _get_spindown(self, device, ignore_missing=False):
+		if device not in self._hdparm_apm_devices:
+			if not ignore_missing:
+				log.info("spindown option is not supported for device '%s'" % device)
+			return None
 		# There's no way how to get current/old spindown value, hardcoding vendor specific 253
 		return 253
 
@@ -361,6 +483,6 @@ class DiskPlugin(hotplug.Plugin):
 		value = self._cmd.read_file(sys_file, no_error=ignore_missing).strip()
 		if len(value) == 0:
 			if not ignore_missing:
-				log.info("disk_scheduler_quantum option is not supported by this HW")
+				log.info("disk_scheduler_quantum option is not supported for device '%s'" % device)
 			return None
 		return int(value)
