@@ -4,7 +4,7 @@ import tuned.logs
 from .functions import functions as functions
 import tuned.consts as consts
 from tuned.utils.commands import commands
-from configobj import ConfigObj, ConfigObjError
+from tuned.utils.config_parser import ConfigParser, Error
 
 log = tuned.logs.get()
 
@@ -40,24 +40,21 @@ class Variables():
 		self._lookup_re[r'(?<!\\)\${' + re.escape(s) + r'}'] = v
 		self._lookup_env[self._add_env_prefix(s, consts.ENV_PREFIX)] = v
 
-	def add_dict(self, d):
-		for item in d:
-			self.add_variable(item, d[item])
-
 	def add_from_file(self, filename):
 		if not os.path.exists(filename):
 			log.error("unable to find variables_file: '%s'" % filename)
 			return
 		try:
-			config = ConfigObj(filename, raise_errors = True, file_error = True, list_values = False, interpolation = False)
-		except ConfigObjError:
+			config = ConfigParser(delimiters=('='), inline_comment_prefixes=('#'), allow_no_value=True, strict=False)
+			config.optionxform = str
+			with open(filename) as f:
+				config.read_string("[" + consts.MAGIC_HEADER_NAME + "]\n" + f.read(), filename)
+		except Error:
 			log.error("error parsing variables_file: '%s'" % filename)
 			return
-		for item in config:
-			if isinstance(config[item], dict):
-				self.add_dict(config[item])
-			else:
-				self.add_variable(item, config[item])
+		for s in config.sections():
+			for o in config.options(s):
+				self.add_variable(o, config.get(s, o, raw=True))
 
 	def add_from_cfg(self, cfg):
 		for item in cfg:
